/* Copyright (c) 1997, MpegTV LLC */
#ifndef _com_api_h
#define _com_api_h
#ifndef lint
static char rcsid_com_api_h[] = "$Header: /net/hopkins/var/home/tristan/mpeg/dec/RCS/com_api.h,v 1.48 1997/08/21 11:51:37 tristan Exp tristan $";
#endif

#include "pmp.h"
#include "alert_api.h"
#include "mode_api.h"
#include "audio_api.h"

#include <X11/Xlib.h>

/*--------------------------------------------------------------------------+
| Configuration flags														|
+--------------------------------------------------------------------------*/

#define COM_API_VERSION	4

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/

/*
 * Commands that can be sent to the Player.
 *
 * Most commands set parameters or ask the player to perform some action.
 * Other command (usually starting with COM_GET_...) ask the player for some data.
 *
 * Everything is quite well tested and has no reasons to change, except
 * the commands marked [experimental].
 *
 * The type of command parameter is indicated first, in parenthesis. (void)
 * means that the command takes no parameter.
 */
typedef enum {		/* (param) description */
	COM_NONE = 0,	/* reserved */
	COM_QUIT = 2,	/* 2: (void) quit */
	COM_OPEN_FILE,	/* 3: (string) read stream from file */
	COM_OPEN_FD,	/* 4: (int) read stream from specified already opened file descriptor */
	COM_OPEN_PORT,	/* 5: (int) read stream from TCP socket (local port # specified) */
	COM_CLOSE_FILE,	/* 6: (void) close stream (close file previously opened by a COM_OPEN...) */
	COM_SAVE_FILE,	/* 7: (string) save copy of stream (if stream is seekable) */
	COM_STATE,		/* 8: (int) set state (param is a STATE) */
	COM_STEP,		/* 9: (void) frame step */
	COM_END,		/* 10: (void) seek to end of input (if stream is seekable) */
	COM_SEEK,		/* 11: (int) seek to specified byte position (if stream is seekable) */
	COM_LOOP,		/* 12: (int) set loop flag (param is a Bool) */
	COM_ZOOM,		/* 13: (int) set zoom (param is a ZOOM_TYPE) */
	COM_MSG_SELECT,	/* 14: (int) set message mask (see msg.h) */
	COM_AUTO_EXIT,	/* 15: (int) set auto_exit flag (param is a Bool) */
	COM_AUTO_PLAY,	/* 16: (int) set auto_play flag (param is a Bool) */
	COM_VIDEO_DITHER_TYPE,	/* 17: (int) set dither type (param is a DITHER_TYPE) */
	COM_VIDEO_GAMMA,/* 18: (double) set gamma correction (param > 0) */
	COM_SYNC_TYPE,	/* 19: (int) set sync type (param is a SYNC_TYPE) */
					/* if param is SYNC_FRAMERATE, enable framerate control
					 * else disable framerate control */
	COM_SYNC_FRAMERATE_SET,	/* 20: (double) set frame-rate (param  > 0) */
	COM_AUDIO_MUTE,	/* 21: (int) set mute flag (param is a Bool) */
	COM_AUDIO_CHANNEL,	/* 22: (int) set audio channel (param is an AUDIO_CHANNEL) */
	COM_AUDIO_QUALITY,	/* 23: (int) set audio quality (param is an AUDIO_QUALITY) */
						/* if param is AUDIO_QUALITY_LOW, disable audio channel control
					 	 * else enable audio channel control */
	COM_AUDIO_VOLUME,	/* 24: (double) set volume (param range is [0.0, 1.0]) */
	COM_AUDIO_BALANCE,	/* 25: (double) set balance (range is [0.0 = LEFT, 1.0 = RIGHT]) */
	COM_AUDIO_CONTROLS,	/* 26: (int) select what volume and balance act on when a mixer is available (param is AUDIO_CONTROLS) */
	COM_WINDOW_CLOSE,	/* 27: (void) close (iconify) player window [not implemented] */
	COM_WINDOW_OPEN,	/* 28: (void) open player window (if iconified) [not implemented] */
	COM_WINDOW_RAISE,	/* 29: (void) raise player window */
	COM_VIDEO_DITHER_COLORMAP_PRIVATE,	/* 30: (int) set private dither colormap flag (param is a Bool) */
	COM_SYS_FLUSH_AFTER_SEEK,	/* 31: (int) tell player to look for the first random access entry point (generally a GOP header), like if a seek had just been performed (param = pos, unused except for indicating offset in error messages) [experimental] */
	COM_SYS_FLUSH_AFTER_NEW_STREAM,	/* 32: (void) tell player to reset to default parsing (i.e. video stream) and to look for the first sequence header [experimental] */
	COM_SYS_READ_NO_DELAY,	/* NOW OBSOLETE! (int) 0 = blocking read, 1 = non-blocking read (on input) */
	COM_OPEN_PORT_UDP,	/* 34: (int) read stream from UDP socket (local port # specified) [experimental] */
	COM_SAVE_IMAGE_PPM,	/* 35: (string) save copy of current frame in ppm format (param = filename) */
	COM_SAVE_IMAGE_TGA,	/* 36: (string) save copy of current frame in tga format (param = filename) */
	COM_GET_POS,		/* 37: (void) get file pointer position */
	COM_GET_ACTUAL_FRAMERATE,	/* 38: (void) get the actual framerate */
	COM_GET_NUMBER_OF_FRAMES_PLAYED,	/* 39: (void) get the number of frames played since last seek */
	COM_AUDIO_MSG_SELECT,	/* 40: (int) set audio process message mask (see msg.h) */
	COM_VIDEO_LUMA_FLOOR,/* 41: (double) set luma floor (i.e. contrast).  negative values increase contrast. */
	COM_VIDEO_LUMA_SHIFT,/* 42: (double) set luma shift (i.e. brightness) */
	COM_AUTO_SEEK,	/* 43: (int) seek to specified byte position right after opening stream (if stream is seekable) [experimental] */
	COM_AUDIO_PORTS_SELECT,	/* 44: (int) select audio output ports (param is a combination of AUDIO_PORT bit masks) */
	COM_WINDOW_NAME,	/* 45: (string) set window name */
	COM_ICON_NAME,		/* 46: (string) set icon name */
	COM_LOOP_START_POS,	/* 47: (int) set byte position of start of loop, or 0 [experimental] */
	COM_LOOP_END_POS,	/* 48: (int) set byte position of end of loop, or 0 [experimental] */
	COM_WINDOW_POSITION_X,	/* 49: (int) set initial horizontal coord. of window (hint for the Window Manager) */
	COM_WINDOW_POSITION_Y,	/* 50: (int) set initial vertical coord. of window (hint for the Window Manager) */
	COM_VIDEO_FREEZE,	/* 51: (int)  0 = play video (default), 1 = freeze video */
	COM_SEND_GOP_INFOS,	/* 52: (int) select whether A2G_COM_GOP_INFOS messages will be received each time a gop is decoded when playing. 0 = no (default), 1 = yes */
	COM_CHALLENGE,		/* reserved */
	COM_X_EVENT_MASK,	/* 54: (int) set window X event mask with XSelectInput() */
	COM_SII_SET,		/* 55: (reserved) */
	COM_SII_LINK,		/* 56: (string) link custom stream input interface */
	COM_SII_UNLINK,		/* 57: (void) unlink custom stream input interface */
	COM_SII_CNTL,		/* 58: (custom) call ci_cntl() routine of custom stream input interface */
	COM_SII_OPEN,		/* 59: (custom) open stream with custom stream input interface */
	COM_MAX				/* marker - follows maximum value defined in this version */
} COMMAND;

/*
 * Types of messages that can be received from the Player.
 *
 * Those message indicate a change of the state of the player, or a change
 * in the parameters of the stream being played, or deliver requested information.
 *
 * Everything is quite well tested and has no reasons to change.
 *
 * The type of message parameter is indicated first, in parenthesis. (void)
 * means that the message has no parameter.
 */
typedef enum {
	A2G_COM_API_VERSION = 1,/* (int) command API version number */
	A2G_COM_RETURN_SUCCESS,	/* (PMP_RETURN) command with PMP_FLAG_RETURN flag set has returned success. A message of this type is always followed by a copy of the original message */
	A2G_COM_RETURN_FAILURE,	/* (PMP_RETURN) command with PMP_FLAG_RETURN flag set has returned failure. A message of this type is always followed by a copy of the original message */
	A2G_COM_READY,			/* (void) player is ready to take commands */
	A2G_COM_QUIT,			/* (void) quit */
	A2G_COM_STATE,			/* (int) player's state changed (param is a STATE) */
	A2G_COM_POS,			/* (int) byte position in stream */
	A2G_COM_ACTUAL_FRAMERATE,/* (double) actual frame rate */
	A2G_COM_NUMBER_OF_FRAMES_PLAYED,/* (int) number of frames played */
	A2G_COM_AUDIO_ENABLED,	/* (void) audio enabled (i.e. audio will be played when playing video) */
	A2G_COM_AUDIO_DISABLED,	/* (void) audio disabled (i.e. audio will not be played when playing video, either because the stream is a video stream or because sync mode or sync frame-rate does not permit audio play) */
	A2G_COM_ALERT,			/* (PMP_DATA_ALERT) alert */
	A2G_COM_AUDIO_VOLUME,	/* (double) new volume (param range is [0.0, 1.0]) */
	A2G_COM_AUDIO_BALANCE,	/* (double) new balance (range is [0.0 = LEFT, 1.0 = RIGHT]) */
	A2G_COM_AUDIO_MUTE,		/* (void) audio has been temporarilly muted by player. This can happen if the driver does not support the selected sampling rate. Application can clear the mute flag with the COM_AUDIO_MUTE command to re-enable audio. */
	A2G_COM_VIDEO_DITHER_TYPE,	/* (int) dither type used (param is a DITHER_TYPE) */
	A2G_COM_VIDEO_DITHER_ENABLE,/* (int) enabled dither type (param is a DITHER_TYPE) */
	A2G_COM_INPUT_FILE_SIZE,/* (int) file size in bytes, or -1 if not seekable */
	A2G_COM_INPUT_FD,		/* (int) file descriptor being read */
	A2G_COM_INPUT_PORT,		/* (int) socket port being read */
	A2G_COM_DISPLAY_READY,	/* (void) display ready (e.g. after seek, etc) */
	A2G_COM_ERROR_IN_STREAM,/* (int) found serious error in stream that triggered concealment (param = reserved) */
	A2G_COM_VIDEO_INFOS,	/* (PMP_DATA_VIDEO_INFOS) new video sequence infos */
	A2G_COM_AUDIO_INFOS,	/* (PMP_DATA_AUDIO_INFOS) new audio infos */
	A2G_COM_FRAME_INFOS,	/* (PMP_DATA_FRAME_INFOS) frame infos */
	A2G_COM_PACK_INFOS,		/* (PMP_DATA_PACK_INFOS) new mux bitrate in bit/sec or 0 if VBR */
	A2G_COM_VIDEO_COLORS_ALLOCATED,	/* (int) number of colors allocated in colormap */
	A2G_COM_AUDIO_PORTS_AVAILABLE,	/* (int) available audio output ports (param is a combination of AUDIO_PORT bit masks) */
	A2G_COM_AUDIO_PORTS_SELECTED,	/* (int) selected audio output ports (param is a combination of AUDIO_PORT bit masks) */
	A2G_COM_SYSTEMS_END_CODE,	/* (void) found a systems-end-code */
	A2G_COM_VIDEO_END_CODE,		/* (void) found a video sequence-end-code */
	A2G_COM_AUDIO_PID,			/* (int) pid of audio process */
	A2G_COM_GOP_INFOS,			/* (PMP_DATA_GOP_INFOS) gop infos */
	A2G_COM_AUDIO_NO_CONTROLS_SELECT,	/* (void) audio mixer not available, COM_AUDIO_CONTROLS not supported */
	A2G_COM_CHALLENGE,			/* (private) authentification request */
	A2G_COM_X_EVENT,			/* (XEvent) XEvent received */
	A2G_COM_SII_FEEDBACK		/* (custom) private feedback message from stream input interface */
} A2G_COM;

/* type of zoom */
typedef enum {
	ZOOM_1X1 = 0,
	ZOOM_2X2
} ZOOM_TYPE;

/* type of video sequence */
typedef enum {
	VIDEO_MPEG1 = 1,
	VIDEO_MPEG2		/* reserved */
} VIDEO_TYPE;

/* video sequence infos */
typedef struct {
	VIDEO_TYPE	type;
	u_int	width;				/* in pels */
	u_int	height;				/* in pels */
	u_int	bitrate;			/* in bytes/sec */
	u_int	vbv_buffer_size;	/* VBV buffer size in bytes */
	double	framerate;			/* in frames/sec */
	Bool	constrained_parameter_flag;
} PMP_DATA_VIDEO_INFOS;

/* temporal direction FORWARD/BACKWARD */
#define F			0	/* FORWARD */
#define B			1	/* BACKWARD */
#define FB			2	/* size of arrays indexed by F and B */

/* spatial direction HORIZONTAL/VERTICAL (used for prediction motion vect) */
#define HT			0	/* HORIZONTAL */
#define VT			1	/* VERTICAL (for MPEG-2 only) */
#define HV			2	/* size of arrays indexed by HT and VT */

/* frame infos */
typedef struct {
	MODE	mode;				/* mode: MODE_INTRA, MODE_PREDICTIVE, MODE_INTERPOLATIVE */
	u_int	temporal_reference;	/* temporal reference */
	double	vbv_delay;			/* vbv_delay (in sec) */
	u_int 	f_code[FB][HV];		/* mc "f_code" [forward, backward][horiz, vert] */
	Bool 	full_pel[FB];		/* if True, motion vect [F, B] are full-pel */
} PMP_DATA_FRAME_INFOS;

/* gop infos */
typedef struct {
	Bool	time_code_drop;		/* time code, drop picture flag */
	u_int	time_code_hours;	/* time code, hours */
	u_int	time_code_minutes;	/* time code, minutes */
	u_int	time_code_seconds;	/* time code, seconds */
	u_int	time_code_pictures;	/* time code, pictures */
	Bool	closed_gop;			/* if True gop is closed */
	Bool	broken_link;		/* if True gop link is broken */
} PMP_DATA_GOP_INFOS;

/* pack infos */
typedef struct {
	u_int	mux_rate;			/* new mux bitrate in bit/sec or 0 if VBR */
} PMP_DATA_PACK_INFOS;

/* param for msg with type A2G_COM_RETURN_SUCCESS or A2G_COM_RETURN_FAILURE */
typedef struct {
	PMP_HEADER	original_header;	/* copy of header of original message */
	int			error;				/* errno (if msg type is A2G_COM_RETURN_FAILURE) */
} PMP_DATA_RETURN;

/* param for msg from player to control (union) */
typedef union {
	PMP_DATA_GENERIC	data_generic;
	PMP_DATA_RETURN	data_return;
	PMP_DATA_ALERT	data_alert;
	PMP_DATA_VIDEO_INFOS data_video_infos;
	PMP_DATA_GOP_INFOS data_gop_infos;
	PMP_DATA_FRAME_INFOS data_frame_infos;
	PMP_DATA_AUDIO_INFOS data_audio_infos;
	PMP_DATA_PACK_INFOS data_pack_infos;
	XEvent				data_x_event;
} PMP_DATA_A2G;

#endif /* not _com_api_h */
